<?php

declare(strict_types=1);

namespace Drupal\schemadotorg_jsonapi_preview\Plugin\Block;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface;
use Drupal\jsonapi_extras\EntityToJsonApi;
use Drupal\schemadotorg_jsonapi\SchemaDotOrgJsonApiManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Schema.org JSON:API preview' block.
 *
 * @Block(
 *   id = "schemadotorg_jsonapi_preview",
 *   admin_label = @Translation("Schema.org Blueprints JSON:API Preview"),
 *   category = @Translation("Schema.org Blueprints")
 * )
 */
final class SchemaDotOrgJsonApiPreviewBlock extends BlockBase implements ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The current route match.
   */
  protected RouteMatchInterface $routeMatch;

  /**
   * The renderer.
   */
  protected RendererInterface $renderer;

  /**
   * The resource type repository.
   */
  protected ResourceTypeRepositoryInterface $resourceTypeRepository;

  /**
   * The entity to JSON:API service.
   */
  protected EntityToJsonApi $entityToJsonApi;

  /**
   * The Schema.org JSON:API manager.
   */
  protected SchemaDotOrgJsonApiManagerInterface $schemaJsonApiManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->routeMatch = $container->get('current_route_match');
    $instance->renderer = $container->get('renderer');
    $instance->resourceTypeRepository = $container->get('jsonapi.resource_type.repository');
    $instance->entityToJsonApi = $container->get('jsonapi_extras.entity.to_jsonapi');
    $instance->schemaJsonApiManager = $container->get('schemadotorg_jsonapi.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'label_display' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(): ?array {
    $entity = $this->getRouteMatchEntity();
    if (!$entity) {
      return NULL;
    }

    // Get includes.
    $resource_type = $this->resourceTypeRepository->get(
      $entity->getEntityTypeId(),
      $entity->bundle()
    );
    $includes = $this->schemaJsonApiManager->getResourceIncludes($resource_type);

    // Retrieve JSON API representation of this node.
    $render_context = new RenderContext();
    $data = $this->renderer->executeInRenderContext($render_context, function () use ($entity, $includes) {
      try {
        return $this->entityToJsonApi->normalize($entity, $includes);
      }
      catch (\Exception $exception) {
        return NULL;
      }
    });

    if (!$data) {
      return NULL;
    }

    // Display the JSON:API using a details element.
    $build = [
      '#type' => 'details',
      '#title' => $this->t('Schema.org JSON:API'),
      '#attributes' => [
        'data-schemadotorg-details-key' => 'schemadotorg-jsonapi-preview',
        'class' => ['schemadotorg-jsonapi-preview'],
      ],
      '#attached' => ['library' => ['schemadotorg_jsonapi_preview/schemadotorg_jsonapi_preview']],
    ];

    // Make the JSON pretty and enhance it.
    $json = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    $build['json'] = [
      '#type' => 'html_tag',
      '#tag' => 'pre',
      '#plain_text' => $json,
      '#attributes' => ['data-schemadotorg-codemirror-mode' => 'application/ld+json'],
      '#attached' => ['library' => ['schemadotorg/codemirror.javascript']],
    ];

    // JSON:API endpoint.
    $entity_type_id = $entity->getEntityTypeId();
    $jsonapi_url = $this->entityToJsonApiUrl($entity, $includes);
    // Allow other modules to link to additional endpoints.
    $build['endpoints'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['schemadotorg-jsonapi-preview-endpoints']],
    ];
    $build['endpoints'][$entity_type_id] = [
      '#type' => 'item',
      '#title' => $this->t('JSON:API endpoint'),
      '#wrapper_attributes' => ['class' => ['container-inline']],
      'link' => [
        '#type' => 'link',
        '#url' => $jsonapi_url,
        '#title' => Unicode::truncate($jsonapi_url->toString(), 255, FALSE, TRUE),
        '#attributes' => ['title' => $jsonapi_url->toString()],
      ],
    ];
    return ['details' => $build];
  }

  /**
   * Return the requested entity's JSON:API URL.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to generate the JSON from.
   * @param array $includes
   *   The list of includes.
   *
   * @return \Drupal\Core\Url
   *   The entity's JSON:API URL.
   *
   * @see \Drupal\jsonapi_extras\EntityToJsonApi::normalize
   */
  protected function entityToJsonApiUrl(EntityInterface $entity, array $includes = []): Url {
    $resource_type = $this->resourceTypeRepository->get(
      $entity->getEntityTypeId(),
      $entity->bundle()
    );
    $route_name = sprintf('jsonapi.%s.individual', $resource_type->getTypeName());
    $route_options = ['absolute' => TRUE];
    if ($resource_type->isVersionable() && $entity instanceof RevisionableInterface && $revision_id = $entity->getRevisionId()) {
      $route_options['query']['resourceVersion'] = 'id:' . $revision_id;
    }
    if ($includes) {
      $route_options['query']['include'] = implode(',', $includes);
    }
    return Url::fromRoute($route_name, ['entity' => $entity->uuid()], $route_options);
  }

  /**
   * Returns the entity of the current route.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity or NULL if this is not an entity route.
   *
   * @see metatag_get_route_entity()
   */
  protected function getRouteMatchEntity(): EntityInterface|NULL {
    $route_name = $this->routeMatch->getRouteName();
    if (preg_match('/entity\.(.*)\.(latest[_-]version|canonical)/', $route_name, $matches)) {
      return $this->routeMatch->getParameter($matches[1]);
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResult {
    return AccessResult::allowedIfHasPermission($account, 'view schemadotorg jsonapi');
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return ['user.permissions', 'url.path'];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $entity = $this->getRouteMatchEntity();
    if ($entity) {
      return Cache::mergeTags(parent::getCacheTags(), $entity->getCacheTags());
    }
    else {
      return parent::getCacheTags();
    }
  }

}
