<?php

declare(strict_types=1);

namespace Drupal\schemadotorg_jsonld_preview\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Routing\AccessAwareRouterInterface;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\schemadotorg_jsonld\SchemaDotOrgJsonLdBuilderInterface;
use Drupal\schemadotorg_jsonld\SchemaDotOrgJsonLdManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Schema.org JSON-LD preview' block.
 *
 * @Block(
 *   id = "schemadotorg_jsonld_preview",
 *   admin_label = @Translation("Schema.org Blueprints JSON-LD Preview"),
 *   category = @Translation("Schema.org Blueprints")
 * )
 */
final class SchemaDotOrgJsonLdPreviewBlock extends BlockBase implements ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The route provider.
   */
  protected AccessAwareRouterInterface $routeProvider;

  /**
   * The router admin context.
   */
  protected AdminContext $routerAdminContext;
  /**
   * The module handler.
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The Schema.org JSON-LD manager service.
   */
  protected SchemaDotOrgJsonLdManagerInterface $schemaJsonLdManager;

  /**
   * The Schema.org JSON-LD builder service.
   */
  protected SchemaDotOrgJsonLdBuilderInterface $schemaJsonLdBuilder;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->routeProvider = $container->get('router');
    $instance->routerAdminContext = $container->get('router.admin_context');
    $instance->moduleHandler = $container->get('module_handler');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->schemaJsonLdManager = $container->get('schemadotorg_jsonld.manager');
    $instance->schemaJsonLdBuilder = $container->get('schemadotorg_jsonld.builder');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'label_display' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(): ?array {
    // Build the entity's Schema.org data.
    $bubbleable_metadata = new BubbleableMetadata();
    $data = $this->schemaJsonLdBuilder->build(NULL, $bubbleable_metadata);
    if (!$data) {
      return NULL;
    }

    // Display the JSON-LD using a details element.
    $build = [
      '#type' => 'details',
      '#title' => $this->t('Schema.org JSON-LD'),
      '#attributes' => [
        'data-schemadotorg-details-key' => 'schemadotorg-jsonld-preview',
        'class' => ['schemadotorg-jsonld-preview', 'js-schemadotorg-jsonld-preview'],
      ],
    ];

    // Apply bubbleable metadata to the render array so that the block is
    // properly cached.
    $bubbleable_metadata->applyTo($build);

    // Attach library after the bubbleable metadata is applied to
    // the render array.
    $build['#attached']['library'][] = 'schemadotorg_jsonld_preview/schemadotorg_jsonld_preview';

    // Make it easy for someone to copy the JSON.
    $t_args = [
      ':schema_href' => 'https://validator.schema.org/',
      ':google_href' => 'https://search.google.com/test/rich-results',
    ];
    $description = $this->t('Please copy-n-paste the below JSON-LD into the <a href=":schema_href">Schema Markup Validator</a> or  <a href=":google_href">Google\'s Rich Results Test</a>.', $t_args);
    $build['copy'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['schemadotorg-jsonld-preview-copy']],
      'description' => [
        '#type' => 'container',
        '#markup' => $description,
      ],
      'button' => [
        '#type' => 'button',
        '#button_type' => 'small',
        '#attributes' => ['class' => ['schemadotorg-jsonld-preview-copy-button', 'button--extrasmall']],
        '#value' => $this->t('Copy JSON-LD'),
      ],
      'message' => [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#attributes' => ['class' => ['schemadotorg-jsonld-preview-copy-message']],
        '#plain_text' => $this->t('JSON-LD copied to clipboard…'),
      ],
    ];

    // JSON.
    // Make the JSON pretty and enhance it.
    $json = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    $build['json'] = [
      'input' => [
        '#type' => 'hidden',
        '#value' => $json,
      ],
      'code' => [
        '#type' => 'html_tag',
        '#tag' => 'pre',
        '#plain_text' => $json,
        '#attributes' => ['data-schemadotorg-codemirror-mode' => 'application/ld+json'],
        '#attached' => ['library' => ['schemadotorg/codemirror.javascript']],
      ],
    ];

    // JSON-LD endpoint.
    // @see schemadotorg_jsonld_endpoint.module
    $entity = $this->schemaJsonLdManager->getRouteMatchEntity();
    if ($entity && $this->moduleHandler->moduleExists('schemadotorg_jsonld_endpoint')) {
      $entity_type_id = $entity->getEntityTypeId();
      $route_name = 'schemadotorg_jsonld_endpoint.' . $entity_type_id;
      $route_parameters = ['entity' => $entity->uuid()];
      $route_options = ['absolute' => TRUE];

      // Make sure the JSON-LD route exists.
      // @see \Drupal\schemadotorg_jsonld_endpoint\Routing\SchemaDotOrgJsonLdEndpointRoutes::routes
      if ($this->routeProvider->getRouteCollection()->get($route_name)) {
        $jsonld_url = Url::fromRoute($route_name, $route_parameters, $route_options);

        // Allow other modules to link to additional endpoints.
        // @see schemadotorg_taxonomy_entity_view_alter()
        $build['endpoints'] = [
          '#type' => 'container',
          '#attributes' => ['class' => ['schemadotorg-jsonld-preview-endpoints']],
        ];
        $build['endpoints'][$entity_type_id] = [
          '#type' => 'item',
          '#title' => $this->t('JSON-LD endpoint'),
          '#wrapper_attributes' => ['class' => ['container-inline']],
          'link' => [
            '#type' => 'link',
            '#url' => $jsonld_url,
            '#title' => $jsonld_url->toString(),
          ],
        ];
      }
    }
    return ['details' => $build];
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResult {
    // If this is an admin route/page never add the JSON-LD to not have
    // JSON-LD impact the admin UI/UX performance.
    // @todo Determine if JSON-LD should be included on admin routes.
    if ($this->routerAdminContext->isAdminRoute()) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowedIfHasPermission($account, 'view schemadotorg jsonld');
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    // Apply the default cache contexts for JSON-LD to ensure that even if the
    // block has not JSON-LD is will be updated if the page now has JSON-LD.
    return array_merge(
      SchemaDotOrgJsonLdBuilderInterface::ROUTE_MATCH_CACHE_CONTEXTS,
      SchemaDotOrgJsonLdBuilderInterface::ENTITY_CACHE_CONTEXTS
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    // Apply the default cache tags for JSON-LD to ensure that even if the
    // block has not JSON-LD is will be updated if the page now has JSON-LD.
    return array_merge(
      SchemaDotOrgJsonLdBuilderInterface::ROUTE_MATCH_CACHE_TAGS,
      SchemaDotOrgJsonLdBuilderInterface::ENTITY_CACHE_TAGS
    );
  }

}
