<?php

declare(strict_types=1);

namespace Drupal\schemadotorg_webpage;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\schemadotorg\SchemaDotOrgEntityFieldManagerInterface;
use Drupal\schemadotorg\SchemaDotOrgMappingInterface;
use Drupal\schemadotorg\SchemaDotOrgMappingManagerInterface;
use Drupal\schemadotorg\SchemaDotOrgNamesInterface;
use Drupal\schemadotorg\SchemaDotOrgSchemaTypeBuilderInterface;
use Drupal\schemadotorg\SchemaDotOrgSchemaTypeManagerInterface;

/**
 * Schema.org WebPage manager.
 */
class SchemaDotOrgWebPageManager implements SchemaDotOrgWebPageManagerInterface {
  use StringTranslationTrait;

  /**
   * Constructs a SchemaDotOrgWebPageManager object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $fieldTypePluginManager
   *   The field type plugin manager.
   * @param \Drupal\schemadotorg\SchemaDotOrgNamesInterface $schemaNames
   *   The Schema.org names service.
   * @param \Drupal\schemadotorg\SchemaDotOrgMappingManagerInterface $schemaMappingManager
   *   The Schema.org mapping manager.
   * @param \Drupal\schemadotorg\SchemaDotOrgSchemaTypeManagerInterface $schemaTypeManager
   *   The Schema.org schema type manager.
   * @param \Drupal\schemadotorg\SchemaDotOrgSchemaTypeBuilderInterface $schemaTypeBuilder
   *   The Schema.org schema type builder.
   */
  public function __construct(
      protected ModuleHandlerInterface $moduleHandler,
      protected ConfigFactoryInterface $configFactory,
      protected EntityTypeManagerInterface $entityTypeManager,
      protected EntityFieldManagerInterface $entityFieldManager,
      protected FieldTypePluginManagerInterface $fieldTypePluginManager,
      protected SchemaDotOrgNamesInterface $schemaNames,
      protected SchemaDotOrgMappingManagerInterface $schemaMappingManager,
      protected SchemaDotOrgSchemaTypeManagerInterface $schemaTypeManager,
      protected SchemaDotOrgSchemaTypeBuilderInterface $schemaTypeBuilder,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function mappingDefaultsAlter(array &$defaults, string $entity_type_id, ?string $bundle, string $schema_type): void {
    // Ignore the WebPage Schema.org type which is not applicable.
    if ($schema_type === 'WebPage') {
      return;
    }

    // Do not set the default for existing mapping.
    /** @var \Drupal\schemadotorg\SchemaDotOrgMappingInterface|null $mapping */
    $mapping = $this->entityTypeManager
      ->getStorage('schemadotorg_mapping')
      ->load("$entity_type_id.$bundle");
    if ($mapping) {
      return;
    }

    // Check that the Schema.org type should default to support the WebPage
    // Schema.org mapping.
    $default_schema_types = $this->configFactory
      ->get('schemadotorg_webpage.settings')
      ->get('default_schema_types');
    if (!$this->schemaTypeManager->isSubTypeOf($schema_type, $default_schema_types)) {
      return;
    }

    // Get the default WebPage Schema.org properties.
    $webpage_mapping_defaults = $this->schemaMappingManager->getMappingDefaults($entity_type_id, NULL, 'WebPage');

    $default_schema_properties = $this->configFactory
      ->get('schemadotorg_webpage.settings')
      ->get('default_schema_properties');

    $schema_properties = array_intersect_key(
      $webpage_mapping_defaults['properties'],
      array_combine($default_schema_properties, $default_schema_properties)
    );

    $webpage_schema_properties = [];
    foreach ($schema_properties as $property_name => $property) {
      $field_name = $property['name'];
      if (empty($field_name) || $field_name === SchemaDotOrgEntityFieldManagerInterface::ADD_FIELD) {
        $field_name = $this->schemaNames->getFieldPrefix() . $property['machine_name'];
      }
      $webpage_schema_properties[$field_name] = $property_name;
    }

    NestedArray::setValue(
      $defaults,
      ['third_party_settings', 'schemadotorg_webpage', 'schema_properties'],
      $webpage_schema_properties
    );
  }

  /**
   * {@inheritdoc}
   */
  public function mappingFormAlter(array &$form, FormStateInterface $form_state): void {
    if (!$this->moduleHandler->moduleExists('schemadotorg_ui')) {
      return;
    }

    /** @var \Drupal\schemadotorg\Form\SchemaDotOrgMappingForm $form_object */
    $form_object = $form_state->getFormObject();
    /** @var \Drupal\schemadotorg\SchemaDotOrgMappingInterface|null $mapping */
    $mapping = $form_object->getEntity();

    // WebPages are only applicable for nodes.
    if ($mapping->getTargetEntityTypeId() !== 'node') {
      return;
    }

    // Exit if no Schema.org type has been selected or if we are currently
    // editing the WebPage Schema.org content type.
    $schema_type = $mapping->getSchemaType();
    if (!$schema_type || $schema_type === 'WebPage') {
      return;
    }

    $mapping_defaults = $form_state->get('mapping_defaults');
    $target_entity_type_id = $mapping->getTargetEntityTypeId();
    $target_bundle = $mapping_defaults['entity']['id'];

    // Get the WebPage mapping defaults.
    $webpage_schema_properties = $this->configFactory
      ->get('schemadotorg_webpage.settings')
      ->get('webpage_schema_properties');
    $webpage_mapping_defaults = $this->schemaMappingManager->getMappingDefaults($target_entity_type_id, NULL, 'WebPage');
    $webpage_mapping_defaults['properties'] = array_intersect_key(
      $webpage_mapping_defaults['properties'],
      array_combine($webpage_schema_properties, $webpage_schema_properties)
    );

    $field_type_definitions = $this->fieldTypePluginManager->getUiDefinitions();

    $field_definitions = $this->entityFieldManager->getFieldDefinitions($target_entity_type_id, $target_bundle);

    $webpage_options = [];
    $webpage_mapping = [];
    foreach ($webpage_mapping_defaults['properties'] as $property_name => $property) {
      $field_name = $property['name'];
      if (empty($field_name) || $field_name === SchemaDotOrgEntityFieldManagerInterface::ADD_FIELD) {
        $field_name = $this->schemaNames->getFieldPrefix() . $property['machine_name'];
      }

      $field_definition = $field_definitions[$field_name] ?? [];
      $field_type = ($field_definition) ? $field_definition->getType() : $property['type'];
      $field_type_definition = $field_type_definitions[$field_type] ?? [];

      $webpage_options[$property_name] = [
        'property' => [
          'data' => [
            '#type' => 'link',
            '#title' => $property_name,
            '#url' => $this->schemaTypeBuilder->getItemUrl($property_name),
          ],
        ],
        'label' => $property['label'],
        'name' => $field_name,
        'type' => $field_type_definition['label'] ?? $property['type'],
        'status' => ($field_definition) ? $this->t('Existing') : $this->t('New'),
        '#attributes' => [
          'class' => ($field_definition) ? 'color-success' : 'color-warning',
        ],
      ];

      $webpage_mapping[$property_name] = $field_name;
    }
    $form_state->set('schemadotorg_webpage_property_mapping', $webpage_mapping);

    $schema_properties = $mapping_defaults['third_party_settings']['schemadotorg_webpage']['schema_properties'] ?? [];
    $form['mapping']['third_party_settings']['schemadotorg_webpage'] = [
      '#type' => 'details',
      '#title' => $this->t('Schema.org WebPage mapping'),
      '#description' => $this->t('Select the Schema.org properties to be mapped to a WebPage Schema.org type.'),
      '#open' => (boolean) $schema_properties,
    ];
    $webpage_schema_subtypes = $this->configFactory
      ->get('schemadotorg_webpage.settings')
      ->get('webpage_schema_subtypes');
    if ($webpage_schema_subtypes) {
      $schema_subtype = $mapping_defaults['third_party_settings']['schemadotorg_webpage']['schema_subtype'] ?? '';
      $form['mapping']['third_party_settings']['schemadotorg_webpage']['schema_subtype'] = [
        '#type' => 'select',
        '#title' => $this->t('WebPage mapping subtype'),
        '#description' => $this->t('Select a WebPage Schema.org subtype that should be applied to this mapping.'),
        '#options' => array_combine($webpage_schema_subtypes, $webpage_schema_subtypes),
        '#empty_option' => $this->t('- Select WebPage subtype -'),
        '#default_value' => $schema_subtype,
      ];
    }
    $form['mapping']['third_party_settings']['schemadotorg_webpage']['schema_properties'] = [
      '#type' => 'tableselect',
      '#title' => $this->t('WebPage mapping properties'),
      '#header' => [
        'property' => $this->t('Schema.org property'),
        'label' => $this->t('Field label'),
        'name' => $this->t('Field name'),
        'type' => $this->t('Field type'),
        'status' => $this->t('Field status'),
      ],
      '#options' => $webpage_options,
      '#default_value' => array_combine($schema_properties, $schema_properties),
    ];

    // Add validation callback.
    $form['#validate'][] = [static::class, 'mappingFormValidate'];
  }

  /**
   * Form submission validation for schemadotorg_webpage_form_schemadotorg_mapping_form.
   *
   * @see \Drupal\schemadotorg_webpage\SchemaDotOrgWebPageManager::mappingFormAlter()
   */
  public static function mappingFormValidate(array &$form, FormStateInterface $form_state): void {
    // Get the third party settings.
    $value = $form_state->getValue(['mapping', 'third_party_settings', 'schemadotorg_webpage']);

    // Filter unchecked properties.
    $value['schema_properties'] = array_filter($value['schema_properties']);

    // Convert the indexed array of Schema.org properties to A mapping with
    // the field name as the key and the Schema.org property as the value.
    $value['schema_properties'] = array_intersect_key(
      $form_state->get('schemadotorg_webpage_property_mapping'),
      array_combine($value['schema_properties'], $value['schema_properties']),
    );

    // Flip the array to make the field name the key and the Schema.org property
    // the value.
    $value['schema_properties'] = array_flip($value['schema_properties']);

    // Remove empty 'schema_properties' so that third_party_settings
    // can be cleared.
    $value = array_filter($value);

    // Update the third party settings.
    $form_state->setValue(['mapping', 'third_party_settings', 'schemadotorg_webpage'], $value);
  }

  /**
   * {@inheritdoc}
   *
   * The below code works around the architecture limitation (or simplicity)
   * which only allows one Schema.org mapping per content type.
   *
   * Therefore, we need to save the mapping as a WebPage with the selected
   * properties, which builds the expected fields, forms, and displays
   * and then revert the mapping back to its original state.
   */
  public function mappingPostSave(SchemaDotOrgMappingInterface $mapping): void {
    if ($mapping->isSyncing()) {
      return;
    }

    // Only apply WebPage mappings to nodes.
    $entity_type_id = $mapping->getTargetEntityTypeId();
    if ($mapping->getTargetEntityTypeId() !== 'node') {
      return;
    }

    // Ignore the WebPage mapping which is not applicable.
    if ($mapping->getSchemaType() === 'WebPage') {
      return;
    }

    $webpage_schema_properties = $mapping->getThirdPartySetting('schemadotorg_webpage', 'schema_properties', []);
    $webpage_schema_properties = array_flip($webpage_schema_properties);
    // Make sure WebPage Schema.org properties are being mapped.
    if (empty($webpage_schema_properties)) {
      return;
    }

    // Get the mapping's settings, which will need to be restored.
    $bundle = $mapping->getTargetBundle();
    $schema_type = $mapping->getSchemaType();
    $schema_properties = $mapping->get('schema_properties');
    $third_party_settings = $mapping->get('third_party_settings');

    // Get the WebPage mapping defaults and use them on the current mapping/bundle.
    $webpage_mapping_defaults = $this->schemaMappingManager->getMappingDefaults($entity_type_id, NULL, 'WebPage');

    // Change the WebPage target bundle.
    $webpage_mapping_defaults['entity']['id'] = $bundle;

    // Clear all third party settings.
    $webpage_mapping_defaults['third_party_settings'] = [];

    // Set the WebPage properties.
    foreach ($webpage_mapping_defaults['properties'] as $schema_property => &$field) {
      $field['name'] = $webpage_schema_properties[$schema_property] ?? NULL;
    }

    // Save the temp WebPage mapping to create expected fields, forms, and displays
    // and get back the updated mapping, which can be reverted.
    $mapping = $this->schemaMappingManager->saveMapping($entity_type_id, 'WebPage', $webpage_mapping_defaults);

    // Restore the original mapping.
    $mapping->set('schema_type', $schema_type);
    $mapping->set('schema_properties', $schema_properties);
    $mapping->set('third_party_settings', $third_party_settings);

    // Re-save the original mapping without syncing enabled.
    $mapping->setSyncing(TRUE);
    $mapping->save();
    $mapping->setSyncing(FALSE);
  }

}
